#ifndef DAQOBJECTS_H_
#define DAQOBJECTS_H_

#include <DAQ++/RunManager.h>
#include <DAQ++/Module.h>
#include <DAQ++/python/pydaqpp.h>
#include <DAQ++/python/pydaqpp_types.h>

/**
 *  The idea is that the python Module holds a pointer to a
 *  DAQpp::Module. Ideally that object is a pyModule, which derives
 *  from DAQpp::Module and hold a pointer to the python object.
*/


class pyModule : public DAQpp::Module, public PyDAQpp
{
    private:
    public:
        pyModule(DAQpp::DAQid const &id)
        : DAQpp::Module(id)
        {
        }

        ~pyModule()
        {
        }

        int check_attributes();

        // Prepare for a new run
        int PrepareForRun();
        // Start the run
        int Start(int mode=0);
        // Stop the run
        int Stop(int mode=0);
        // Pause the run
        int Pause();
        // Resume the run after pause
        int Continue();
        // Send a reset to the module
        int Reset();
        // Initialize the Module
        int GetReady();
        // Issues a trigger
        int Trigger();
        // Flush any remaining data on the Module
        void flush();
        // Tells if the module has data available
        bool hasData();
        // Returns true if the Module has data available
        char *getData(int &size);

        bool get_monitor_data(int &sz, char **dt);
        void reset_monitor_data();
};


class pyRunManager : virtual public DAQpp::RunManager, public PyDAQpp
{
    public:
        pyRunManager(DAQpp::DAQid const &id) :
            DAQpp::RunManager(id)
        {
        }
        ~pyRunManager()
        {
        }

        int check_attributes();

        void PrepareForRun();
        void EndOfRun();
        void EndOfEvent();
        int writeData(int size, const char *data);
};



#endif /*DAQOBJECTS_H_*/
