/*
 * clientParams.h
 *
 *  Created on: 29 Dec 2010
 *      Author: lacasta
 */

#ifndef CLIENTPARAMS_H_
#define CLIENTPARAMS_H_

#include <string>
#include <vector>
#include <algorithm>

/**
 * A collection of routines to get/set parameters from the client side
 */
template<typename Type, typename anyType>
Type get_par_value(ns1__Parameter *P)
{
    return ((anyType *)P->value->value->item[0])->__item;
}

template<typename Type, typename anyType>
void get_vpar_value(ns1__Parameter *P, std::vector<Type> &out)
{
    out.clear();
    if ( P->value->value->item.empty() )
        return;

    ns1__valueType *vp = (ns1__valueType *)P->value->value->item[0];
    std::vector< xsd__anyType *> &values = vp->item;
    std::vector< xsd__anyType *>::iterator ip;
    for (ip=values.begin(); ip!=values.end(); ip++)
        out.push_back(  ((anyType *)*ip)->__item   );
}

template<typename Type, typename anyType>
anyType *instantiate_type(soap *__ss, Type value)
{
    anyType __hack;
    struct soap_clist *cp = soap_link(__ss, NULL, __hack.soap_type(), -1, soap_fdelete);

    anyType *__any = new anyType();
    cp->ptr = (void *)__any;
    __any->soap = __ss;
    __any->__item = value;
    return __any;
}

template<typename Type, typename anyType>
ns1__Parameter *set_parameter(soap *ss, const std::string &name, Type val)
{

    ns1__Parameter *out = soap_new_ns1__Parameter(ss, -1);
    out->value = soap_new_ns1__ParameterValue(ss, -1);
    out->value->value = soap_new_ns1__valueType(ss, -1);

    anyType *any = instantiate_type<Type, anyType>(ss, val);
    out->value->name = name;
    out->value->value->item.push_back( any );

    return out;
}

template<typename Type, typename anyType>
struct inserter
{
        soap *ss;
        inserter(soap *s) : ss(s) {}
        anyType *operator()(Type v)
        {
            anyType *any =  instantiate_type<Type, anyType>(ss, v);
            return any;
        }
};

template<typename Type, typename anyType>
ns1__Parameter *set_vparameter(soap *__ss, const std::string &name, std::vector<Type> &val)
{
    ns1__Parameter *out = soap_new_ns1__Parameter(__ss, -1);
    out->value = soap_new_ns1__ParameterValue(__ss, -1);
    out->value->value = soap_new_ns1__valueType(__ss, -1);
    out->value->name = name;

    ns1__valueType *any = soap_new_ns1__valueType(__ss, -1); //new ns1__valueType();
    any->item.insert(any->item.end(), val.size(), (xsd__anyType *)0);
    std::transform(val.begin(), val.end(), any->item.begin(), inserter<Type, anyType>(__ss));
    out->value->value->item.push_back( any );

    return out;
}



#endif /* CLIENTPARAMS_H_ */
