""" This provides a series of helpers to access the DAQ++ soap server
"""
import os, sys
import time
import re
import cPickle
import base64
from SOAPpy import SOAPProxy
from SOAPpy import WSDL
import SOAPpy
import traceback

#SOAPpy.Config.debug = 1
DAQPP_NS="http://localhost/daqpp/"

def load_wsdl():
    """ Loads the wsdl file from the server and prints 
        information about it
    """
    url = 'http://localhost:50000/daqpp'
    
    # just use the path to the wsdl of your choice
    wsdlObject = WSDL.Proxy(url + '?wsdl')
    print 'Available methods:'
    for method in wsdlObject.methods.keys() :
        print method
        ci = wsdlObject.methods[method]
        # you can also use ci.inparams
        for param in ci.outparams :
            # list of the function and type 
            # depending of the wsdl...
            print param.name.ljust(20) , param.type
        print
        
# modify SOAPBuilder
def dump_ParameterValue(self, obj, tag, typed = 1, ns_map = {}):
    """ This is a helper function for SOAPpy that will construct
        the request for a ParameterValue.
    """
    if SOAPpy.Config.debug: print "In dump_ParameterValue. tag=", tag
    tag = tag or self.gentag()
    tag = SOAPpy.wstools.XMLname.toXMLname(tag) # convert from SOAP 1.2 XML name encoding

    id = self.checkref(obj, tag, ns_map)
    if id == None:
        return
    
    try: a = obj._marshalAttrs(ns_map, self)
    except: a = ''

    try:
        ns = ns_map[DAQPP_NS]
        tnm = 'xsi:type="%s:ParameterValue"' % (ns)
    except KeyError:
        tnm = ''
    
    self.out.append('<%s %s>\n' % (tag, tnm))
    self.out.append('<name>%s</name>\n' % obj.name)
    self.out.append('<type>%s</type>\n' % obj.type)
    
    self.dump(obj.value, "value", 1, ns_map)
    
    self.out.append('</%s>\n' % tag)
    
def dump_valueType(self, obj, tag, typed = 1, ns_map = {}):
    """ Helper function for SOAPpy to create a request for a 
        valueType
    """
    if SOAPpy.Config.debug: print "In dump_valueType. tag=", tag
    tag = tag or self.gentag()
    tag = SOAPpy.wstools.XMLname.toXMLname(tag) # convert from SOAP 1.2 XML name encoding

    id = self.checkref(obj, tag, ns_map)
    if id == None:
        return
    
    try: a = obj._marshalAttrs(ns_map, self)
    except: a = ''
        

    try:
        ns = ns_map[DAQPP_NS]
        tnm = 'xsi:type="%s:valueType"' % (ns)
    except KeyError:
        tnm = ''
    
    
    self.out.append('<%s %s>\n' % (tag, tnm))
    
    if obj.type == "_object*":
        self.out.append('<item xsi:type="xsd:base64Binary" xmlns:xsd="http://www.w3.org/1999/XMLSchema">%s</item>\n' % base64.b64encode(obj.item[0]))
    
    else:
        is_array = (len(obj.item)>1)
        if is_array:
            self.out.append('<item %s>\n' % (tnm))
    
        for item in obj.item:    
            self.dump(item, "item", 1, ns_map)
        
        if is_array:
            self.out.append('</item>\n')
        
    self.out.append('</%s>\n' % tag)
        
def dump_int(self, obj, tag, typed = 1, ns_map = {}):
    """ SOAPpy insists in using std::integer for the int, while we use xsd:int
        This will change the way in which SOAPbuilder does the job
    """
    if SOAPpy.Config.debug: print "In my_dump_int."
    self.out.append(self.dumper(None, 'int', obj, tag, typed,
                                     ns_map, self.genroot(ns_map)))

def dump_SOAPParam( self, obj, tag, typed = 1, ns_map = {} ):
    """ Helper function for SOAPpy to construct a Parameter request.
    """
    if SOAPpy.Config.debug: print "In dump_valueType. tag=", tag
    tag = tag or self.gentag()
    tag = SOAPpy.wstools.XMLname.toXMLname(tag) # convert from SOAP 1.2 XML name encoding

    id = self.checkref(obj, tag, ns_map)
    if id == None:
        return
    
    try: a = obj._marshalAttrs(ns_map, self)
    except: a = ''
    
    ns = ns_map[DAQPP_NS]
    try:
        ns = ns_map[DAQPP_NS]
        tnm = 'xsi:type="%s:Parameter"' % (ns)
    except KeyError:
        tnm = ''
    
    self.out.append('<param %s>\n' % (tnm))
    self.out.append('<daqid>%s</daqid>\n' % obj.daqid)    
    self.dump(obj.value, "value", 1, ns_map)
    self.out.append('</param>\n')
    

class valueType(object):
    """ This class represents a valueType object.
        This is a generic type used in the DAQ++ soap implementation.
        The actual object is in the `item` attribute
    """
    type_list = {"short":int,
                 "int": int,
                 "long": long,
                 "unsigned short": int,
                 "unsigned int": int,
                 "unsigned long": long,
                 "float": float,
                 "double": float,
                 "string": str,
                 "std::string": str,
                 }
    def __init__(self, value, type):
        cast = valueType.type_list.get(type, None)
        self.type = type
        if cast is None and type == "_object*":
            sss = cPickle.dumps(value)
            self.item = (sss, )
            return
        
        try:
            if self.type=="string" or self.type=="std::string":
                self.item = (cast(value),)
                
            else:
                hack = value[0]
                self.item = [ cast(x) for x in value ]
                
        except TypeError:
            self.item = (cast(value),)
        

class ParameterValue(object):
    """ Represents a Parameter value.
        It contains the name of the parameter, its type and value (as a valueType)
    """
    vvector = re.compile("std::vector<(?P<type>\w+).*>")
    
    def __init__(self, name, value, type):
        self.name = name
        ss = ParameterValue.vvector.search(type)
        if ss:
            self.type = ss.group("type")
        else:
            self.type = type
            
        self.value = valueType(value, self.type)
        
class SOAPParam(object):
    """ This represents a SOAP Parameter type. It contains the daqid
        of the owner and the value as a Parameter value
    """
    def __init__(self, daqid, name, value, type):
        self.daqid = daqid
        self.value = ParameterValue(name, value, type)

class Parameter(object):
    """ This represents a parsed Parameter. The main attributes are:
        - name: the parameter name
        - obj: the daqid of the parameter owner 
        - type: the type of the parameter
        - value: the actual value
    """
    def __init__(self, name, obj, server=None, input=None):
        self.name = name
        self.obj = obj
        self.server = server
        self.type = None
        self.is_vector = None
        self.value = None
        if self.server is not None:
            self.value = self.get_value() 
        elif input:
            self.value = self.decode(input)
        
    def __str__(self):
        return "Parameter %s@%s: %s" %(self.name, self.obj, self.value)
        
    def get_value(self):
        """ Gets the parameter value from the server
            @return: the value of the parameter
        """
        if self.server is None:
            return None
        
        par = self.server.getParameter(parID={"holderID":self.obj, "parName":self.name })
        return self.decode(par)
        
    
    def set_value(self, value):
        """ Sets the parameter value in the server
            @param value: the new value 
        """
        if self.server is None:
            return
        
        self.server.setParameter( SOAPParam(self.obj, self.name, value, self.type) )
        
    def decode(self, par):
        """ decode(self, par)
            Helper function to parse the Parameter as returned by the
            SOAP server.
            @param par: the answer of the server 
        """
        if not self.type:
            self.type = par.value.type
            if self.type == "std::string":
                self.type = "string"
                
            self.is_vector = (self.type.find("std::vector") >= 0)

        if self.is_vector:
            return par.value.value.item.item
        else:
            try:
                if self.type == "_object*":
                    import cPickle
                    return cPickle.loads( par.value.value.item )
                else:
                    if (isinstance(par.value.value, str)):
                        return par.value.value
                    else:
                        return par.value.value.item
            except:
                print "Exception in user code:"
                print '-'*60
                print 'Parameter: ', par.value.name
                traceback.print_exc(file=sys.stdout)
                print '-'*60
                return None            

class Status(object):
    """ The status of a DAQObject.
    """
    attr_list = {'status': lambda x:x, 
                 'nwritten': int, 
                 'name': lambda x:x, 
                 'daqid' : lambda x:x,
                 'ntrigger': int, 
                 'rate': float, 
                 'nexpected': int, 
                 'throughput': float, 
                 'time': float, 
                 'eff': float}
    def __init__(self, response):
        for attr, cast in Status.attr_list.items():
            setattr(self, attr, cast(getattr(response, attr)))
            
    def __str__(self):
        rate = self.rate;
        rate_units=" Hz";
        thr_units = " Mb/s";

        if  rate>1.e6:
            rate /=1.e6
            rate_units="MHz"
   
        elif rate>1000.0: 
            rate /=1000.
            rate_units = "kHz"
   
        throughput = self.throughput/self.time;
        if throughput > 1048576.0:
            throughput /= 1048576.0
            thr_units = "Mb/s"
        elif throughput > 1024.0:
            throughput /= 1024.0
            thr_units = "kb/s"
        elif throughput > 1.0:
            thr_units = "b/s"
        else:
            throughput *= 1024
            thr_units = " mb/s"

        out = "Evts %7d time %6.1f rate %8.1f %s eff %5.1f - %7.1f %s" % \
            (self.ntrigger, self.time, rate, rate_units, self.eff, throughput, thr_units)

        return out

class DAQObject(object):
    """ This is a DAQObject
    """
    def __init__(self, name, server):
        self.server = server
        self.name = name
        self.parameters = {}
        
        par = server.getParameterNames(daqid=name)
        try:
            if isinstance(par.name, str):
                par.name = (par.name,)
                
            for name in par.name:
                self.parameters[name] = Parameter(name, self.name, server)
        except AttributeError:
            pass
        
    def getStatus(self):
        """ Gets the status from the server
            @return: the status of the DAQObject
        """
        try:
            res = self.server.getStatus(daqid=self.name)
            return Status(res)
        except Exception, e:
            print e
            return None
        
    def getMonitorData(self):
        """ Gets the monitor data from the server
            @return: the monitor data. It is up to the user to decode this.
        """
        rc = self.server.getMonitorData( daqid=self.name )
        return rc
    
    def resetMonitorData(self):
        """ Resets the monitor data in the server
        """
        self.server.resetMonitorData( daqid=self.name )

class RunManager(DAQObject):
    """ A RunManager
    """
    def __init__(self, name, server):
        super(RunManager, self).__init__(name, server)
        
    def startRun(self):
        """ Starts a run for this RunManager
        """
        self.server.StartRunManager(daqid=self.name)
        
    def stopRun(self):
        """ Stops the run
        """
        self.server.StopRunManager(daqid=self.name)
        
    def GetReady(self):
        self.server.GetReady(daqid=self.name)
        
    def Pause(self):
        self.server.Pause(daqid=self.name)
        
    def Continue(self):
        self.server.Continue(daqid=self.name)
        
    def Reset(self):
        self.server.Reset(daqid=self.name)
        
    def setMaxEvents(self, max_events):
        """ Sets the maximum number of events for the run
            @param max_events: the max. number of events. If <=0, it will 
                               be an infinite run.
        """
        self.server.setMaxEvents(maxEvents={"daqid":self.name, "nevts":max_events})


def parse_update(out):
    """ parse_update(out)
        Parses the output of the sendUpdate command.
        @param out: the ouput of the sendUpdate command.
        @return: a tuple (status, parameters) where status is a list of the 
                 most recent status changes and parameters a list of the
                 recently modified parameters
    """
    status = []
    parameters = []
    
    # Get the list of status
    sl = out['status']
    if isinstance(sl, str):
        # empty, do nothing
        pass
    elif isinstance(sl.status, list):
        for st in sl.status:
            status.append(Status(st))
    else :
        status.append(Status(sl.status))
        
        
    pl = out['parameters']
    if isinstance(pl, str):
        # empty, do nothing
        pass
    elif isinstance(pl.parameter, list):
        for par in pl.parameter:
            parameters.append( Parameter(par.value.name, par.daqid, input=par) )
    else:
        par = pl.parameter
        parameters.append( Parameter(par.value.name, par.daqid, input=par) )
        

    return status, parameters


if hasattr(SOAPpy.Config, "dumpmap"):
    SOAPpy.Config.dumpmap = (  (int, dump_int),  
                               (ParameterValue, dump_ParameterValue),
                               (valueType, dump_valueType),
                               (SOAPParam, dump_SOAPParam)
                            )
else:
    def funcToMethod(func,clas,method_name=None):
        """ Helper function to add dynamically methods to a class
        """
        import new
        method = new.instancemethod(func,None,clas)
        if not method_name: method_name=func.__name__
        clas.__dict__[method_name]=method
    
    funcToMethod(dump_ParameterValue, SOAPpy.SOAPBuilder, "dump_ParameterValue")
    funcToMethod(dump_valueType, SOAPpy.SOAPBuilder, "dump_valueType")
    funcToMethod(dump_SOAPParam, SOAPpy.SOAPBuilder, "dump_SOAPParam")
    funcToMethod(dump_int, SOAPpy.SOAPBuilder, "dump_int")


