/* -*- mode: c++ -*- */
#ifndef __vmedaq_module_h__
#define __vmedaq_module_h__

#include <vector>
#include <analysis/PadMap.h>

/**
 * This class is a dummy type to store Module configurations
 */
class ModuleConfig
{
    public:
        ModuleConfig() {}
        virtual ~ModuleConfig() {}
        
        
        virtual int load_config(const char *fnam) {return 0;}
        virtual int dump_config(const char *fnam) { return 0; }

};

/**
 * This class encapsulates the information about a Module.
 */
class Module
{
    private:
        /// the module ID
        int _id;

        /// This is the data type stored in the description of the
        /// module in the start of run
        int _type;

        /// Number of chips
        int _nchip;

        /// Number of channels per chip
        int _nchan;

        /// Total number of channels
        int _ntot;

        /// The pixel map
        PadMap *_pad_map;

        /// The module configuration
        ModuleConfig *_config;
        
        /// Pedestals
        std::vector<double> _pedestals;

        /// Noise
        std::vector<double> _noise;

    public:
        Module(int id, int nchip, int type = 0);
        virtual ~Module();

        int id() const
        {
            return _id;
        }
        void id(int x)
        {
            _id = x;
        }

        int type() const
        {
            return _type;
        }
        void type(int x)
        {
            _type = x;
        }

        int nchip() const
        {
            return _nchip;
        }
        void nchip(int x)
        {
            _nchip = x;
        }

        int nchan() const
        {
            return _nchan;
        }
        void nchan(int x)
        {
            _nchan = x;
        }

        int ntot() const
        {
            return _ntot;
        }
        void ntot(int x)
        {
            _ntot = x;
        }

        const PadMap *pad_map() const
        {
            return _pad_map;
        }
        void pad_map(const PadMap &p)
        {
            if (!_pad_map)
            {
                _pad_map = new PadMap(p);
            }
            else
            {
                *_pad_map = p;
            }
        }

        
        ModuleConfig *config()
        {
            return _config;
        }

        const ModuleConfig *config() const
        {
            return _config;
        }
        
        /**
         * Sets the module configuration. THe class instace becomes
         * responsible of deleting the pointer
         */
        void set_config( ModuleConfig *C);
        
        std::vector<double> &pedestals()
        {
            return _pedestals;
        }
        const std::vector<double> &pedestals() const
        {
            return _pedestals;
        }
        double pedestal(int x) const
        {
            return _pedestals[x];
        }

        std::vector<double> &noise()
        {
            return _noise;
        }
        const std::vector<double> &noise() const
        {
            return _noise;
        }
        double noise(int x) const
        {
            return _noise[x];
        }
};

#endif
