/* -*- mode: c++ -*- */
#ifndef __PadMap_h__
#define __PadMap_h__

#include <iosfwd>
/**
   \brief class PadMap
   \ingroup CONFIG

   This is a helper class that stores a pad-channel mapping.
   Loops first on columns (x)  and then on rows (y)
*/
class PadMap
{
private:
    int _nx, _ny; //number of cells in X and Y respectively
    int _ntot;
    int  *_pad2chan;
    int  *_chan2pad;
    char *_connected;

    void init();
    void clear();
    void cpy(const PadMap &m);
public:
    PadMap(int nx, int ny, int ntot = -1);
    PadMap(const char *fname, int nx, int ny);
    PadMap(int n, int *m, int nx, int ny);
    PadMap(const PadMap &m);
    PadMap &operator=(const PadMap &m);
    ~PadMap();
    int  nx() const
    {
        return _nx;
    }
    int  ny() const
    {
        return _ny;
    }
    int  ntot() const
    {
        return _ntot;
    }

    // Unconnected channels return 0xff for ix and/or iy
    void pad(int channel, int &ix, int &iy) const
    {
        if ( channel >= 0 && channel < _ntot )
        {
            if (  _chan2pad[channel] == -1 )
            {
                ix = 0xff;
                iy = 0xff;
            }
            else
            {
                ix = _chan2pad[channel] % _nx;
                iy = _chan2pad[channel] / _nx;
            }
        }
        else
        {
            ix = 0xff;
            iy = 0xff;
        }
    }

    int get_pad_number(int channel) const
    {
        return _chan2pad[channel];
    }
    int get_chan_number(int pad) const
    {
        return _pad2chan[pad];
    }

    int channel(int x, int y) const
    {
        int ipad = x + _nx * y;
        if ( ipad >= 0 && ipad < _ntot && _connected[ipad])
        {
            return _pad2chan[ipad];
        }
        else
            return 0;
    }
    void set(int channel, int x, int y);

    void print_unconnected() const;
    void print(std::ostream &os) const ;
};
inline std::ostream &operator<<(std::ostream &os, const PadMap &pm)
{
    pm.print(os);
    return os;
}

#endif
