/* -*- mode: c++ -*- */
#ifndef __GeneralConfig_h__
#define __GeneralConfig_h__
#include <config/HstColor.h>
#include <config/SectionParser.h>


/**
   \brief class General
   \ingroup CONFIG

   Stores general settings. This needs a bit of refactoring... Starts
   looking like a big bag...
*/
class GeneralConfig : public SectionParser
{
    public:
        /// pedestal method
        enum PedestalMethod { fast, normal };
        

    private:
        /// Number of events in a pedestal run
        int ped_nevts;

        /// Do pedestal update
        bool ped_update;

        /// Pedestal Method
        PedestalMethod ped_method;

        /// Minimum number of events in a channel to trust the pedestal
        int ped_min_sample;

        /// Pedestals node
        xmlNodePtr ped_node;

        /// Period to dump pedestals in file
        int ped_period;

        /// Corba: use IP (if false uses unix sockets)
        bool corba_use_ip;

        /// Corba: port (it we use IP, this is the port number)
        int corba_port;

        /// Corba node
        xmlNodePtr corba_node;

        /// DAQ node
        xmlNodePtr daq_node;

        /// monitor buffer
        int monitor_buffer;

        /// VME driver
        std::string vme_lib;
        std::string vme_factory;

        // colors for histograms and tracers
        HstColor tracer_color, hist_color;
        
        /// used to by the copy constructor
        void cpy(const GeneralConfig &g);
    public:
        /// Default constructor
        GeneralConfig();

        /// Copy constructor
        GeneralConfig(const GeneralConfig &g)
        {
            cpy(g);
        }

        /// Destructor
        ~GeneralConfig()
        {}

        void parse_node(xmlNodePtr )
            throw (ConfigException);

        
        void parse_colors(xmlNodePtr );
        void save_colors(xmlNodePtr );
        
        void save_config() 
            throw (ConfigException);

        int get_ped_nevts() const
        {
            return ped_nevts;
        }
        void set_ped_nevts( int x )
        {
            ped_nevts = x;
        }

        bool get_ped_update() const
        {
            return ped_update;
        }
        void set_ped_update(bool x)
        {
            ped_update = x;
        }

        PedestalMethod get_ped_method() const
        {
            return ped_method;
        }
        void set_ped_method(PedestalMethod x)
        {
            ped_method = x;
        }

        int get_ped_min_sample() const
        {
            return ped_min_sample;
        }
        void set_ped_min_sample(int x)
        {
            ped_min_sample = x;
        }

        int get_ped_period() const
        {
            return ped_period;
        }
        void set_ped_period(int x)
        {
            ped_period = x;
        }

        int get_monitor_buffer() const
        {
            return monitor_buffer;
        }
        void set_monitor_buffer(int x)
        {
            monitor_buffer = x;
        }

        void decode_pedestals(xmlNodePtr);

        bool get_corba_use_ip() const
        {
            return corba_use_ip;
        }
        void set_corba_use_ip( bool x )
        {
            corba_use_ip = x;
        }
        int  get_corba_port() const
        {
            return corba_port;
        }
        void set_corba_port(int x)
        {
            corba_port = x;
        }
        
        std::string get_vme_lib() const 
        { 
            return vme_lib; 
        }
        
        std::string get_vme_factory() const
        {
            return vme_factory;
        }
        
        const HstColor &tracers() const { return tracer_color; }
        void tracers( const HstColor &c) { tracer_color = c; }

        const HstColor &hsts() const { return hist_color; }
        void hsts( const HstColor &c) { hist_color = c; }
};


#endif
