/* -*- mode: c++ -*- */
#ifndef __PathConfig_h__
#define __PathConfig_h__

#include <config/SectionParser.h>
#include <config/SearchPath.h>
#include <vector>
#include <string>

/**
   \brief class PathConfig.

   \ingroup CONFIG

   Stores information on extra paths defined by the user. VMEDAQ can
   llok into some of those, like for instance \em glade that will be
   used to override the standard glade file.

*/

class PathConfig : public SectionParser
{
    public:
        enum PathType { libpath, gladepath, configpath, pythonpath, none };
        //typedef std::vector<std::string> PathList;
        
    private:
        
        /// Map to store all the paths
        typedef std::map<PathType, SearchPath > PathMap;
        PathMap paths;
        
        /// Used to copy or define
        void cpy(const PathConfig &s);

    public:
        // Default constructor
        PathConfig();

        /// Copy constructor
        PathConfig(const PathConfig &s)
        {
            cpy(s);
        }
        /// Asignment operator
        void operator=(const PathConfig &s)
        {
            if (this != &s)
                cpy(s);
        }
        /// Destructor
        ~PathConfig()
        {
            paths.clear();
        }

        void parse_node(xmlNodePtr) throw (ConfigException);
        void parse_block(PathType type, xmlNodePtr );
        void save_config() throw (ConfigException);
        void save_block(PathType type, const std::string &xpath);
        
        void add_path(PathType type, const std::string &p);
        void add_path(PathType type, const SearchPath &P);
        void set_path(PathType type, const SearchPath &P);
        SearchPath get_path(PathType type);
        
        /// Finds a library
        static std::string find_library(const std::string &lib);
        
        /// Finds a glade file
        static std::string find_glade_file(const std::string &file);
        
};
#endif
