#ifndef UTILS_H_
#define UTILS_H_
#include <cstring>
#include <string>
#include <utility>
#include <algorithm>
/**
 * Splits a string into tokens separated by any of the given 
 * separators.
 */
template <typename Container>
void stringtok (Container &container, std::string const &in,
                const char * const delimiters = " \t\n")
{
    const std::string::size_type len = in.length();
    std::string::size_type i = 0;

    while ( i < len )
    {
        // eat leading whitespace
        i = in.find_first_not_of (delimiters, i);
        if (i == std::string::npos)
            return;   // nothing left but white space

        // find the end of the token
        std::string::size_type j = in.find_first_of (delimiters, i);

        // push token
        if (j == std::string::npos)
        {
            container.push_back (in.substr(i));
            return;
        }
        else
            container.push_back (in.substr(i, j - i));

        // set up for next loop
        i = j + 1;
    }
}

extern int (*__flower)(int);
extern int (*__fupper)(int);

inline std::string downcase(const std::string &s)
{
    std::string str = s;
    transform(str.begin(), str.end(), str.begin(), __flower);
    return s;
}
inline std::string uppercase(const std::string &s)
{
    std::string str = s;
    transform(str.begin(), str.end(), str.begin(), __fupper);
    return s;
}

inline bool isws(char c, char const * const wstr=" \t\n")
{
    return (strchr(wstr, c) != NULL);
}

inline std::string trim_right(const std::string &s)
{
    std::string b=" \t\n";
    std::string str = s;
    return str.erase(str.find_last_not_of(b) +1);
}
inline std::string trim_left(const std::string &s)
{
    std::string b=" \t\n";
    std::string str = s;
    return str.erase( 0, str.find_first_not_of(b) );
}
inline std::string trim_str(const std::string &s)
{
    std::string str = s;
    return trim_left(trim_right(str) );
}

std::pair<std::string, std::string> split_path(const std::string);
#endif /*UTILS_H_*/
