#ifndef _NEWPOINT_H_
#define _NEWPOINT_H_

#include <DAQ++/GTimer.h>

/** \ingroup DAQ
 * This is a class that controls if we need to change to a new
 * scan point.
 */
class NewPoint
{
    private:
        bool started;
    public:
        /// Constructor
        NewPoint() : started(false)
        {}
        virtual ~NewPoint()
        {}
        // Returns true if we need to start a new point
        virtual bool operator()() =0;

        // Resets the state
        virtual void reset()
        {}

        // "increments" the state
        virtual void next()
        {}

        bool has_started() const
        {
            return started;
        }
        void start()
        {
            started = true;
        }
	    /** TODO: find a better way to implement this */
	    virtual void set_max_value(double val) {}
	    virtual double get_max_value() const { return 0.0; }
};

/** \ingroup DAQ
 * Changes from one point to the next based on a fixed number
 * of events per point in the scan
 */
class EventCntr : public NewPoint
{
    private:
        /// Total number of event
        int nevts;

        /// Current event
        int ievt;
    public:
        /// Constructor
        EventCntr(int n) : nevts(n), ievt(0)
        {}

        /// Destructor
        virtual ~EventCntr()
        {}

        /// returns true if we need to change to a new point
        bool operator()()
        {
            return (ievt % nevts) == 0;
        }

        /// resets the current event
        void reset()
        {
            ievt = 0;
        }
        void next()
        {
            ievt++;
        }
	void set_max_value(double val) {
	  nevts = int(val);
	}
	double get_max_value() const { return nevts; }
};

/** \ingroup DAQ
 *  Changes from one point in the scan to the next based on
 * a fixed ammount of time spent on each point
 */
class EventTimer : public NewPoint
{
    private:
        /// Maximum time for a scan point
        double mxtime;

        /// A timer
        DAQpp::GTimer timer;
    public:
        /// Constructor
        EventTimer(double mx) : mxtime(mx)
        {
            timer.reset();
            timer.stop();
        }

        /// Destructor
        ~EventTimer()
        {}

        /// returns true if we need to change to a new point
        bool operator()();
        void reset()
        {
            timer.reset();
            timer.start();
        }
	void set_max_value(double val) {
	  mxtime = val;
	}
	bool is_active()
	{
	  return timer.is_active();
	}
	double get_max_value() const { return mxtime; }

};


#endif /*_NEWPOINT_H_*/
