#ifndef __PedestalRunManager_h__
#define __PedestalRunManager_h__

#include <daq/Rmanager.h>
#include <daq/IntTrgRunManager.h>

/** \ingroup DAQ
 * 
 * This RunManager is prepared to acquire data to compute the
 * pedestals and noise of the different channels.
 * 
 */
class PedestalRunManager : public IntTrgRunManager
{
    public:
        /**
         * Constructor.
         * 
         * This RunManager runs without Monitor and DataLogger and the 
         * constructor makes sure that this is so.
         */
        PedestalRunManager(DAQpp::DAQid id);
        
        /** Destructor.
         */
        ~PedestalRunManager();

        /**
         * Prepares the modules for a pedestal run.
         * 
         * The RunManager will reset the monitor data of each Module and
         * will tell their corresponding Analysis to update the pedestals.
         */
        void PrepareForRun();
        
        /**
         * Instead of logging the data on file, it will force the call to 
         * Analysis::Fill so that all the events are used for pedestal
         * calculation
         */
        int writeData(int size, const char *data);
        
        
        /**
         * Loads the pedestals in the Analysis into the module.
         */
        void EndOfRun();
};

#endif

