#ifndef __DefaultMonitorGUI_included__
#define __DefaultMonitorGUI_included__
#include <iostream>
#include <iomanip>
#include <sstream>
#include <set>

#include <DAQ++/DAQid.h>
#include <data/Analysis.h>
#include <gui/WidgetUpdate.h>
#include <gui/GladeApp.h>

//////////////////////////////////////////////////////////////////////////
// class Monitor
//
//   Created: Mon May 27 17:42:33 2002        Author: Carlos Lacasta
//   Purpose:
//
//////////////////////////////////////////////////////////////////////////
class DefaultMonitorGUI : public GladeApp
{
    private:
        DAQpp::DAQid _id;
        int tmout, tmout_stats;
        int g_chip;
        Analysis *_ana;
        bool updating;
        bool has_stats_prop;

        typedef std::set<GtkWidget *> HstList;
        HstList hlist;

        static void on_Quit(GtkWidget *w, DefaultMonitorGUI *m);
        static void on_Reset(GtkWidget *w, DefaultMonitorGUI *m);
        static void on_ResetPed(GtkWidget *w, DefaultMonitorGUI *m);
        static void on_Help(GtkWidget *w, DefaultMonitorGUI *m);
        static void on_NewChip(GtkWidget *, DefaultMonitorGUI *m);
        static void on_ChangeView(GtkWidget *, DefaultMonitorGUI *m);
        static void on_Change2Noise(GtkWidget *, DefaultMonitorGUI *m);
        static void on_Btn2DSignal(GtkToggleButton *, DefaultMonitorGUI *m);
        static void on_Btn2DMap(GtkWidget *, DefaultMonitorGUI *m);
        static void on_BtnApplyCut(GtkWidget *, DefaultMonitorGUI *m);
        static void on_BtnResetCut(GtkWidget *, DefaultMonitorGUI *m);
        static gboolean on_WindowSelDown(GtkWidget *, GdkEventButton *, DefaultMonitorGUI *);
        static gboolean on_WindowSelUp(GtkWidget *, GdkEventButton *, DefaultMonitorGUI *);
        static int on_Update(void *);
        static int on_UpdateStats(void *);

        static GtkWidget *create_hst_pedestals(DefaultMonitorGUI *, char *, char *, char *, int,
                                               int);
        static GtkWidget *create_hst_noise(DefaultMonitorGUI *, char *, char *, char *, int, int);
        static GtkWidget *create_hst_signal(DefaultMonitorGUI *, char *, char *, char *, int, int);
        static GtkWidget *create_event_display(DefaultMonitorGUI *, char *, char *, char *, int,
                                               int);
        static GtkWidget *create_hitmap(DefaultMonitorGUI *, char *, char *, char *, int, int);
        static GtkWidget *create_tracer_cmmd(DefaultMonitorGUI *, char *, char *, char *, int, int);

        static GtkWidget *create_gauge_rate(DefaultMonitorGUI *, char *, char *, char *, int, int);
        static GtkWidget *create_gauge_eff(DefaultMonitorGUI *, char *, char *, char *, int, int);

        GtkWidget *create_2D_frame(const std::string &prefix, char *xtit1, char *ytit1,
                                   char *xtit2, char *ytit2);

        void stop_histograms();
        void update_histograms();
        void reset_histograms();

        void toggle2Dhst(const char *frame, GtkToggleButton *);
        void resize_hst1d(GtkWidget *, int, double, double, gpointer, double hmin = 0.,
                          double hmax = 0.);
        void
                resize_hst2d(GtkWidget *, int, double, double, int, double, double,
                             gpointer);
    public:
        DefaultMonitorGUI(const DAQpp::DAQid &id, const std::string &g_file, Analysis *a);
        ~DefaultMonitorGUI();

        void inLocal();
        void inGlobal();

        // GladeApp methods
        void user_init();
        void auto_connect();
        void set_handlers();
};

#endif
