/* -*- mode: c++ -*- */
#ifndef __GladeApp_h__
#define __GladeApp_h__

#include <string>
#include <vector>
#include <map>
#include <gtk/gtk.h>
#include <glade/glade.h>

#define SET_CONNECT(fnc) connect[#fnc]=(GCallback)fnc

/**
 * \ingroup GUI
 * \brief   class GladeApp.
 * 
 * This class is the core of the interface with libglade.
 */
class GladeApp
{
    protected:
        typedef GtkWidget * (*GladeHandler)(GladeApp *, gchar *name, gchar *s1,
                                            gchar *s2, int i1, int i2);
    private:
        typedef std::map<std::string, std::string> ValueMap;
        typedef std::map<std::string, GCallback> CallBackMap;
        typedef std::map<std::string, GladeHandler> HandlerMap;
        typedef std::map<int, GObject *> TextSet;

        GladeXML *xml;
        GtkWidget *window;

        /*
         * Opens the Glade file
         */
        void open_xml_file(const char *, const char *, const char *);

        static void connect_callback(const char *handler, GObject *obj,
                                     const char *signam, const char *sig_data,
                                     GObject *connect_object, gboolean after,
                                     gpointer data);

        static GtkWidget *handler(GladeXML *_xml, gchar *func_name, gchar *name,
                                  gchar *string1, gchar *string2, gint int1, gint int2,
                                  gpointer user_data);

        
    protected:
        CallBackMap connect;
        HandlerMap handlers;
        ValueMap values;
        TextSet ctext;

        static void on_text_changed(GtkWidget *, gpointer data);
        static gboolean on_enter_notify(GtkWidget *, GdkEventFocus *, gpointer);
        static gboolean on_leave_notify(GtkWidget *, GdkEventFocus *, gpointer);
        void block_signals();
        void unblock_signals();
        
    public:
        GladeApp();
        virtual ~GladeApp();

        int init(const char *fnam, const char *root = 0, const char *domain = 0);
        GtkWidget *get_main_window()
        {
            return window;
        }
        GtkWidget *get_widget(const std::string &s) const;

        char *get_file_name() const
        {
            return xml->filename;
        }

        // This is called after the glade file is loaded
        virtual void user_init()
        {
        }

        // The user should fill connect in this routine
        virtual void auto_connect()
        {
        }
        ;

        // The user should fill handlers in this routine
        virtual void set_handlers()
        {
        }
        ;

        // Any GtkEditable that changes its text will trigger a call to this
        virtual void process_text(const std::string &s, GtkWidget *w)
        {
        }

        /**
         * Add a folder or list of folders (separated by any of ":;,|")
         * to the list of folders where glade files are searched
         */
        static void add_path(const std::string &s);
        static std::string find_file(const std::string &s);

        std::string get_text(const std::string &s) const;
        void set_text(const std::string &wname, const std::string &the_text);
        
        int get_int(const std::string &s, int def);
        double get_double(const std::string &s, double def);

        static std::string double2str(double x, char *fmt = 0);
        static std::string int2str(long x, char *fmt = 0);

        bool btn_active(const std::string &s);
        void activate_button(const std::string &s, bool state);
        void click_button(const std::string &s);

        int entry2int(const std::string &s, int def = 0);
        double entry2double(const std::string &s, double def = 0.);

        void int2entry(const std::string &s, int, char *fmt = 0);
        void double2entry(const std::string &s, double, char *fmt = 0);
        
        int spin2int(const std::string &s);
        double spin2double(const std::string &s);

        void int2spin(const std::string &s, int);
        void double2spin(const std::string &s, double);

};

#endif
