/* -*- mode: c++ -*- */
#ifndef __MonitorWindow_h__
#define __MonitorWindow_h__
#include <vector>
#include <gtk/gtkwidget.h>
#include <gui/WidgetUpdate.h>

/** \ingroup GUI
 *  \brief   Container of the monitor window.
 *
 *  This is the class that the ModuleParser will return when the
 *  monitor method is called. It should contain the pointer to the
 *  widget representing the monitor window and also a list of
 *  WidgetUpdate objects that will need to be updated.
 *
 *  It has, in addition, the methods inLocal and inGlobal that will
 *  tell the monitor window when the module is in local or global
 *  mode.
 */
class MonitorWindow
{
    private:
        std::vector<WidgetUpdate *> _updates;

    protected:
        GtkWidget *_widget;

    public:
        MonitorWindow(GtkWidget *w = 0) :
            _widget(w)
        {
        }
        virtual ~MonitorWindow();


        void add_update(WidgetUpdate *u)
        {
            _updates.push_back(u);
        }

        GtkWidget *widget() const
        {
            return _widget;
        }
        const std::vector<WidgetUpdate *> &updates() const
        {
            return _updates;
        }

        virtual void inLocal()
        {
        }
        virtual void inGlobal()
        {
        }
};

#endif
