#!/usr/bin/env python
""" These are DAQ objects as seen from the client side.
    It implements the xmlModule and the xmlRunManager
"""
import base64
import array

class xmlStatus(object):
    """ It stores the status
    """
    def __init__(self, values):
        for name, val in values.iteritems():
            object.__setattr__(self, name, val)
            
    def __str__(self):
        rate = self.rate
        rate_units = "Hz"
        
        if rate > 1.e6:
            rate /= 1.e6
            rate_units = "MHz"
        elif rate > 1000.:
            rate /= 1000.
            rate_units = "kHz"
            
            
        s = "Evts %7.0f time %6.1f rate %8.1f %s eff %5.1f - %7.1f Mb/s [%s]" % (
                self.ntrig, self.time, rate, rate_units, self.eff, self.throughput, self.status)
        return s

class xmlDObject(object):
    """ This class is common to all the DAQObjects
    """
    def __init__(self, server, daqid):
        """ The class will store the server and the 
            DAQid of the object
        """
        self.server = server
        self.id = daqid
        self.par_names = None #self.server.get_par_names(daqid)
        
    def get_par(self, name):
        """ Gets the value of the parameter with name 'name'
        """
        rc = None
        try:
            rc = self.server.get_par(self.id, name)
        except Exception, ex:
            print "Error getting parameter",name,"\n", ex
            
        return rc
        
    def set_par(self, name, val, notify=False):
        """ Sets the value of parameter 'name'
        """
        self.server.set_par(self.id, name, val, notify)
        
    def get_par_names(self):
        """ Returns the list of defined parameters
        """
        #return self.par_names
        return self.server.get_par_names(self.id)
    
    def get_status(self):
        """ Returns the status of the object
        """
        status = self.server.get_status(self.id)
        return xmlStatus(status)

    def add_par(self, name, value):
        """ Adds a new parameter
        """
        return self.server.add_par(self.id, name, value)

class xmlModule(xmlDObject):
    """ A DAQ++ xmlModule
    """
    def __init__(self, server, daqid):
        super(xmlModule, self).__init__(server, daqid)
        
    def reset_mon_data(self):
        return self.server.reset_mon_data(self.id)
    
    def get_mon_data(self):
        out = None
        data = self.server.get_mon_data(self.id)
        if data:
            ss = base64.b64decode(data)
            out = array.array('c', ss)
            
        return out
        
class xmlRunManager(xmlDObject):
    """ A DAQ++ xmlRunManager
    """ 
    def __init__(self, server, daqid):
        super(xmlRunManager, self).__init__(server, daqid)

    def is_active(self):
        """ Tells if the RUnManager is active
        """
        return self.server.runm_is_active(self.id)
    
    def activate(self):
        """ Activates the xmlRunManager
        """
        return self.server.runm_active(self.id)
    
    def Start(self):
        """ Starts the xmlRunManager
        """
        return self.server.start_runm(self.id)
    
    def Stop(self):
        """ Stops the xmlRunManager
        """
        return self.server.stop_runm(self.id)
    
    def GetReady(self):
        """ Send a GetReady command
        """
        return self.server.getready_runm(self.id)
    
    def Pause(self):
        """ Pauses the run
        """
        return self.server.pause_runm(self.id)
    
    def Continue(self):
        """ Resumes the run
        """
        return self.server.resume_runm(self.id)
    
    