""" 
    This module implements a DataPipe and an EventBuilder
    as explained in the vmedaq manual
"""

import vmedaq

class DataPipe(object):
    """ This is a data pipe. It has a data container that maps
        the module id with the data of the current event
    """
    def __init__(self):
        """ Initialization
        """
        self.data = {}
        
    def clear_data(self):
        """ Clears all the data in preparation for a new event
        """
        self.data.clear()
        
    def add_data(self, id, data):
        """ Adds the data relative to the given module id 
        """
        self.data[id] = data
        
    def analyze_event(self, evt_num):
        """ Analyzes the event at the end of the event
        """
        pass
    
class EventBuilder(vmedaq.SimpleVMEdaqObserver):
    """ This is the event builder. It has a DataPipe, detects end
        and begin of events and calls the analyze_event of the DataPipe
    """
    def __init__(self, pipe=None):
        """ Initialization. Accepts as argument a new DataPipe. 
            If not given, creates a new one
        """
        super(EventBuilder, self).__init__(0xffffffffL)
        if not pipe or not isinstance(pipe, DataPipe):
            self._pipe = DataPipe()
        else:
            self._pipe = pipe
            
    def get_pipe(self):
        """ Returns an instance of the pipe
        """ 
        return self._pipe
            
    def append_data(self, module, evtnum, data):
        """ Adds new data to the pipe
        """ 
        self._pipe.add_data(module.id, data)
        
    
    def EndOfEvent(self, id, evtnum):
        """ Called at the end of an event
        """
        self._pipe.analyze_event(evtnum)
        self._pipe.clear_data()        
        