/* -*- mode: c++ -*- */
#ifndef __ADCdef_h__
#define __ADCdef_h__

/*******************************************************************************/
/*                    V775   REGISTERS RELATIVE ADDRESSES                      */
/*******************************************************************************/
#define OUTPUT_BUFFER 0x0000   /*    */

#define FIRMWARE        0x1000      /*    */
#define GEO_ADDR        0x1002      /*    */
#define MCST_ADDR       0x1004      /*    */
#define BITSET1         0x1006      /*    */
#define BITCLEAR1       0x1008      /*    */
#define INT_LEV         0x100A      /*    */
#define INT_VECT        0x100C      /*    */
#define STATUS1         0x100E      /*    */
#define CONTROL1        0x1010      /*    */
#define ADER32          0x1012
#define ADER24          0x1014
#define SSHOT_RES       0x1016
#define MCST_CONTROL    0x101A
#define SW_BERR         0x101C




#define TRG_EVNT        0x1020
#define STATUS2         0x1022      /*    */
#define EVNT_CNT_L      0x1024      /*    */
#define EVNT_CNT_H      0x1026      /*    */
#define SKIP_EVNT        0x1028      /*    */
#define INC_DWORD       0x102A      /*    */
#define NLOAD           0x102C      /*    */
#define RANGE           0x102E      /*    */

#define BITSET2         0x1032      /*    */
#define BITCLEAR2       0x1034      /*    */
#define TEST_ADDR       0x1036      /*    */
#define TESTWORD_H      0x1038      /*    */
#define TESTWORD_L      0x103A      /*    */
#define CRATE           0x103C
#define TEST_EVNT_WRITE 0x103E
#define EVNT_CNT_RES    0x1040      /*    */



/*#define EVNT_REQD     0x1030*/    /*    */

#define VSET            0x1060
#define VOFF            0x1062
#define R_TEST_ADDR     0x1064      /*    */
#define RST_TIME        0x1066
#define COMSET          0x1068
#define SLIDE_CNST      0x106A
#define CAL_SET         0x106C
#define CAL_CLEAR       0x106E
#define AAD             0x1070
#define BAD             0x1072


/*config. ROM*/
#define OUI0            ROM(0x0026)
#define OUI1            ROM(0x002A)
#define OUI2            ROM(0x002E)
#define BOARDID0        ROM(0x0032)
#define BOARDID1        ROM(0x0036)
#define BOARDID2        ROM(0x003A)
#define BOARDID3        ROM(0x003E)
#define MB_REVIS        ROM(0x004E)
#define PB_REVIS        ROM(0x0052)
#define SERNUM1         ROM(0x0F02)
#define SERNUM0         ROM(0x0F06)
#define FIGUREA         ROM(0x0008)
#define WIPERA          ROM(0x000A)
#define FIGUREB         ROM(0x000C)
#define WIPERB          ROM(0x000E)
#define PRIMO           ROM(0x0000)
#define SECONDO         ROM(0x0002)
#define TERZO           ROM(0x0004)
#define QUARTO          ROM(0x0006)



#define ROM(r)          (0x8000 + ((r) & 0x7FFF)) /* ROM address is 15 bit wide*/
#define AUXBUS(r)       (0x1200 + (2*(r) & 0x00FF)) /* AUXBUS address is 8 bit wide*/

/*-----------------------------------------------------------------------------*/
/*                    ADDRESSES OF THE 32 THRESHOLDS REGISTERS                 */
/*-----------------------------------------------------------------------------*/


#define     THRES(r)    (0x1080+2*(r))
/*-----------------------------------------------------------------------------*/
/*                    ADDRESSES OF THE AUXILIARY DAC BOARD(LANDI)                   */
/*-----------------------------------------------------------------------------*/

#define DAC_ADD 0xCCCC0018 /*DAC voltage*/
#define NumStep 10         /*numero step*/
#define V785_8V_Step 9     /*numero step*/
#define NumRange 5         /*numero range*/

/*-----------------------------------------------------------------------------*/
/*                    ADDRESSES OF THE V678 REGISTERS                          */
/*-----------------------------------------------------------------------------*/

#define V678_ADD 0x33000000 /*V678 base address*/

#define Out_En_Reg 0x00 /*V678 Output Enable Register Offset*/
#define Control_Reg 0x02 /*V678 Control Register Offset*/
#define Timing_Reg 0x04 /*V678 Timing Register Offset*/

/*******************************************************************************/
/*                  MACROS  FOR SINGLE BIT OR BIT PAIR SELECTION                       */
/*******************************************************************************/

/*-----------------------------------------------------------------------------*/
/*                 STATUS1 REGISTER                                            */
/*-----------------------------------------------------------------------------*/

#define EVNTTRG_BIT(r)       (((r) & 0x0100) >> 8)     /*  bit 8 selection*/
#define TERM_BITS(r)         (((r) & 0x00C0) >> 6)     /*  bit pair [7,6] selection*/
#define TERMOFF_BIT(r)       (((r) & 0x0080) >> 7)     /*  bit 7 selection*/
#define TERMOK_BIT(r)        (((r) & 0x0040) >> 6)     /*  bit 6 selection*/
#define AMNESIA_BIT(r)       (((r) & 0x0010) >> 4)     /*  bit 4 selection */
#define GLOBAL_BUSY_BIT(r)   (((r) & 0x0008) >> 3)     /*  bit 3 selection */
#define LOCAL_BUSY_BIT(r)    (((r) & 0x0004) >> 2)     /*  bit 2 selection*/
#define GLOBAL_DTRDY_BIT(r)  (((r) & 0x0002) >> 1)     /*  bit 1 selection*/
#define LOCAL_DTRDY_BIT(r)   (((r) & 0x0001) >> 0)     /*  bit 0 selection*/

/*----------------------------------------------------------------------------*/
/*                 STATUS2 REGISTER                                           */
/*----------------------------------------------------------------------------*/

#define RDMODE(r)           (((r) & 0x300) >> RDMOD0_ORD) /*READ MODE: 00 = FIFO; OTHERS NOT DEFINED YET*/
#define PBTYPE(r)           (((r) & 0x00F0) >> 4)     /*  bits[7:4] selection*/
#define FULL_BIT(r)         (((r) & 0x0004) >> 2)     /*  bit 2 selection*/
#define EMPTY_BIT(r)        (((r) & 0x0002) >> 1)     /*  bit 1 selection */
#define EVNT_AVAIL_BIT(r)   (((r) & 0x0001) >> 0)     /*  bit 0 selection*/
#define BUFF_BIT(r)         (((r) & 0x0006) >> 1)     /*  bit pair [2,1] selection*/

/*---------------------------------------------------------------------------*/
/*                 CONTROL1 REGISTER                                         */
/*---------------------------------------------------------------------------*/

#define BERREN_BIT(r)      (((r) & 0x0020) >> 5)  /*  bit 5 selection        */
#define PRES_MODE_BIT(r)   (((r) & 0x0010) >> 4)  /*  bit 4 selection*/

#define BLKEND_BIT(r)      (((r) & 0x0004) >> 2)  /*  bit 2 selection        */


/*----------------------------------------------------------------------------*/
/*                 BITSET1 (BITCLEAR1) REGISTER                               */
/*----------------------------------------------------------------------------*/

#define RESET_MODE_BIT(r)       (((r) & 0x0080) >>7)       /* bit 7 selection: reset mode*/

#define RELOC_BIT(r)            (((r) & 0x0010) >>4)       /* bit 4 selection: relocation*/
#define BERR_FLAG_BIT(r)        (((r) & 0x0008) >>3)       /* bit 3 selection: berr flag*/



/*----------------------------------------------------------------------------*/
/*                 BITSET2 (BITCLEAR2) REGISTER                               */
/*----------------------------------------------------------------------------*/
#define TRG_CNT_BIT(r)          (((r) & 0x4000) >>14)      /* bit 14 selection*/
#define SLIDE_ZR_BIT(r)         (((r) & 0x2000) >>13)      /* bit 13 selection*/
#define EMPTY_PROG_BIT(r)       (((r) & 0x1000) >>12)      /* bit 12 selection*/
#define AUTO_INCR_BIT(r)        (((r) & 0x0800) >>11)      /* bit 11 selection*/
#define SS_BIT(r)               (((r) & 0x0400) >>10)      /* bit 10 selection*/
#define RDMOD1_BIT(r)           (((r) & 0x0200) >>9)        /* bit 9 selection*/
#define RDMOD0_BIT(r)           (((r) & 0x0100) >>8)        /* bit 8 selection*/
#define SLIDEN_BIT(r)           (((r) & 0x0080) >>7)        /* bit 7 selection*/
#define TEST_ACQ_BIT(r)         (((r) & 0x0040) >>6)        /* bit 6 selection*/
#define VALPROG_BIT(r)          (((r) & 0x0020) >>5)        /* bit 5 selection*/
#define UPPROG_BIT(r)           (((r) & 0x0010) >>4)        /* bit 4 selection*/
#define OVRPROG_BIT(r)          (((r) & 0x0008) >>3)        /* bit 3 selection*/
#define CLRDATA_BIT(r)          (((r) & 0x0004) >>2)        /* bit 2 selection*/
#define OFFLINE_BIT(r)          (((r) & 0x0002) >>1)        /* bit 1 selection*/
#define TESTON_BIT(r)           (((r) & 0x0001) >>0)        /* bit 0 selection*/

/******************************************************************************/
/*                             other macros                                   */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/*                            splitted  data interpretation                   */
/*----------------------------------------------------------------------------*/

#define DATA_EVNT_CNT(r)       ((r) & 0xFFFFFF)
#define DATA_GOOD_CHANNELS(r)  (((r)>>8)  & 0x3F)
#define DATA_CRATENUM(r)       (((r)>>16) & 0xFF)
#define DATA_TYPE(r)           (((r)>>25) & 0x3)    /*01 HEAD   00 DATUM  10 EOB*/
#define DATA_GEO(r)            (((r)>>27) & 0x1F)
#define DATA_ADC(r)            ((r)       & 0xFFF)
#define DATA_OVERTH(r)         (((r)>>12) & 0x1)
#define DATA_DWTH(r)           (((r)>>13) & 0x1)
#define DATA_VAL(r)            (((r)>>14) & 0x1)
#define DATA_CH(r)             (((r)>>16) & 0x1F)

/*----------------------------------------------------------------------------*/
#define MCST_POS(r)            ((r) & 0x3)


/*----------------------------------------------------------------------------*/
/*                         MACROS FOR SINGLE BIT OPERATIONS                   */
/*----------------------------------------------------------------------------*/

#define TEST_BIT(r,b)   ( ((r) & (1<<(b))) >> (b) )
#define SET_BIT(r,b)    r = (ushort)((r) | (1<<(b)))
#define CLEAR_BIT(r,b)  r = (ushort)((r) & (~(1<<(b))))
#define TOGGLE_BIT(r,b) r = (ushort)((r) ^ (1<<(b)))

/*----------------------------------------------------------------------------*/
/*                         OTHER MACROS                                       */
/*----------------------------------------------------------------------------*/
#define VOLTAGE(p)  (0x100 + (p) * 0x15e)

/******************************************************************************/
/*                              USEFUL CONSTANTS                              */
/******************************************************************************/

#define BA_32      0
#define BA_24      1
#define GEO_24     2
#define BUFF_SIZE  256
#define NOT_VALID  2
#define TAMB       0
#define RELOC      1

#define HEADER   1
#define EOB      2
#define DATUM    0
#define GARBAGE  3

#define PB_TAC   0

#define LAST     1
#define FIRST    2
#define ACTIVE   3
#define INACTIVE 0

/******************************************************************************/
/*                             ORD OF ALL OF THE REGISTERS BITS               */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/*                 STATUS1 REGISTER                                           */
/*----------------------------------------------------------------------------*/


#define TERMOFF_ORD    7
#define TERMOK_ORD     6
#define AMNESIA_ORD    4
#define GLOB_BUSY_ORD  3
#define LOC_BUSY_ORD   2
#define GLOB_DTRDY_ORD 1
#define LOC_DTRDY_ORD  0


/*----------------------------------------------------------------------------*/
/*                 STATUS2 REGISTER                                           */
/*----------------------------------------------------------------------------*/

#define DSEL0_ORD      4
#define FULL_ORD       2
#define EMPTY_ORD      1
#define EVNT_AVAIL_ORD 0

/*----------------------------------------------------------------------------*/
/*                 CONTROL1 REGISTER                                          */
/*----------------------------------------------------------------------------*/

#define BERREN_ORD 5
#define PRG_RS_ORD 4
#define BLKEND_ORD 2



/*----------------------------------------------------------------------------*/
/*                 BITSET1 (BITCLEAR1) REGISTER                               */
/*----------------------------------------------------------------------------*/

#define RESET_MODE_ORD  7
#define BER_DIS_ORD     6
#define RELOC_ORD       4
#define BERR_FLAG_ORD   3


/*-------------------------------------------------------------------
 *                  BITSET2 (BITCLEAR2) REGISTER
 *-----------------------------------------------------------------*/

#define TRGCNT_ORD     14
#define SLIDEZR_ORD    13
#define EMPTY_PROG_ORD 12
#define AUTO_INCR_ORD  11
#define SS_ORD         10
#define RDMOD1_ORD     9
#define RDMOD0_ORD     8
#define SLIDEN_ORD     7
#define TEST_ACQ_ORD   6
#define VALPROG_ORD    5
#define UPPROG_ORD     4
#define OVRPROG_ORD    3
#define CLRDATA_ORD    2
#define OFFLINE_ORD    1
#define TESTON_ORD     0

/*-------------------------------------------------------------------
 *                                  CALIBRATE SET
 * ----------------------------------------------------------------*/
#define UN1_ORD     6
#define UN0_ORD     5
#define UPOT_ORD    3
#define INCPOT_ORD  2
#define BPOT_ORD    1
#define APOT_ORD    0

#define COM_ORD 0

/*-------------------------------------------------------------------
 *                                  conversion sequence
 * ----------------------------------------------------------------*/
#define SEQUENCE(board,i) ((((board)!=874) && ((board)!=878)) ? (((i)/2) + (((i)%2) * 16)):(i)  )
/*for all boards except 874: 0,16,2,17...
 * for others: i*/



#endif
