/* -*- mode: c++ -*- */
#ifndef __ADCmodule_h__
#define __ADCmodule_h__
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <vector>
#include <daq/GenModule.h>

class ADCmodule : public GenModule
{
    public:
        enum ADCThreshold
        {   Low, High, Disable};

    private:
        std::vector<int> thrs;
        ADCThreshold thrs_type;
    public:
        ADCmodule(const DAQpp::DAQid &id, unsigned long add);
        ~ADCmodule();

        bool hasData();
        char *getData(int &size);
        int PrepareForRun();
        int Start(int = 0);
        int Stop(int = 0)
        {
            return 0;
        }
        ;
        int Pause()
        {
            return 0;
        }
        ;
        int Continue()
        {
            return 0;
        }
        ;
        int Reset();
        int GetReady();
        int Trigger()
        {
            return 0;
        }
        ;
        void flush()
        {
        }
        ;

        int get_thres(int x) const
        {
            return thrs[x] & 0xff;
        }
        void set_threshold(int chan, int val)
        {
            int xval = (val & 0xff);
            thrs[chan] = (thrs[chan]&0x100) | xval;
        }
        void set_mask(int chan, bool val)
        {
            if (val)
                thrs[chan] |= 0x100;
            else
                thrs[chan] &= 0xff;
        }
        bool get_mask(int x) const
        {
            return (thrs[x]&0x100)!=0;
        }
        void read_threshold();
        void write_threshold();
        void set_threshold(const std::vector<int> &v);
        const std::vector<int> &get_threshold() const
        {
            return thrs;
        }
        void set_mask(const std::vector<bool> &v);

        unsigned long get_address() const
        {
            return _add;
        }
        unsigned long get_data_type() const;
        void set_header_description(unsigned int &data);
        unsigned int *write_config(int &size);

        void reset_monitor_data();

        ADCThreshold get_thrs_type() const
        {
            return thrs_type;
        }
        void set_thrs_type(ADCThreshold tp);
};

#endif
