/* -*- mode: c++ -*- */
#ifndef __CSprint_h__
#define __CSprint_h__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <vector>
#include <daq/GenModule.h>
#include <fSio.h>
#define NCHAN 20

struct ADCRecord
{
    unsigned long id;         //  'CSPT'
    unsigned char version;
    unsigned char reclen;     // # of bytes in record
    unsigned char status;
    unsigned char CSR;
    unsigned short wordcount; // number of events left (we won't use this)
    unsigned short counter;   // event counter (16 bits)
    unsigned char boardID;    // ID (8 bit switch) of this CSPRINT ADC board
    unsigned char preamp3;    // 20 bits to enable/disable individual preamps
    unsigned char preamp2;
    unsigned char preamp1;
    unsigned short adc[NCHAN];   // 20 adc words, 12 bits
    unsigned short adcsum;    // energy sum adc
};

class CSPRINT : public GenModule
{
private:
    fSio usb;
public:
    CSPRINT( const DAQpp::DAQid &id, const  char *);
    ~CSPRINT();

    bool hasData();
    char *getData(int &size);
    int PrepareForRun ();
    int Start (int = 0);
    int Stop (int = 0)
    {
        return 0;
    };
    int Pause ();
    int Continue ()
    {
        return 0;
    };
    int Reset ();
    int GetReady ();
    int Trigger ()
    {
        return 0;
    };
    void flush ();

    unsigned long get_address() const
    {
        return _add;
    }
    unsigned long get_data_type() const;
    void set_header_description(unsigned int &data);
};


#endif
