/* -*- mode: c++ -*- */
#ifndef __FBTDCdef_h__
#define __FBTDCdef_h__

/* ----------------------------------------
   FRITZ Slave Module  Address Map

   FRITZ Board Base Address
   FRITZ is hardwired to address 0x1000000 in A32/D32 VME space.
   The mapping of this space in CPU address space can only only
   be resolved at runtime by a call to SysBusToLocalAdrs  (in sysLib) since
   CPU boards differ greatly.
*/
#define FRITZ_BASE 0x1000000
#define FRITZ_AM   0xD

/* (some) FRITZ PIO Chip Addresses */

#define PIO_BASE ((unsigned int)(FRITZ_BASE + 0x100000))

#define PIO_PRIM_AD_CYCLE ((unsigned int)(PIO_BASE + 0x10))
#define PIO_DAT_CYCLE ((unsigned int)(PIO_BASE + 0x14))
#define PIO_SET_EG_MS ((unsigned int)(PIO_BASE + 0x18))
#define PIO_RD_WT_SS ((unsigned int)(PIO_BASE + 0x1c))
#define PIO_CLR_AS ((unsigned int)(PIO_BASE + 0x20))

/*
  LeCroy Fastbus TDC 1877 Definitions
*/

/* Model */
#define LECROY1877_ID ((unsigned int) 0x103c0000)
#define LECROY1877S_ID ((unsigned int) 0x103d0000)

#define LECROY1877_CSR0  ((unsigned int) 0x0)
#define LECROY1877_CSR1  ((unsigned int) 0x1)
#define LECROY1877_CSR3  ((unsigned int) 0x3)
#define LECROY1877_CSR5  ((unsigned int) 0x5)
#define LECROY1877_CSR7  ((unsigned int) 0x7)
#define LECROY1877_CSR16  ((unsigned int) 0x10)
#define LECROY1877_CSR18  ((unsigned int) 0x12)

/*********************************************************/
/* Define bits in CSR0                                   */
/*********************************************************/
#define LECROY1877_CLEAR    ((unsigned int) 0x80000000)
#define LECROY1877_RESET    ((unsigned int) 0x40000000)
#define LECROY1877_INTERCOM ((unsigned int) 0x800000)
#define LECROY1877_MCONFIG  11 /* TEST ME!!!! */
#define LECROY1877_LNE      ((unsigned int) 0x400)
#define LECROY1877_SSPARSE  ((unsigned int) 0x200)
#define LECROY1877_PRIMING  ((unsigned int) 0x100)
#define LECROY1877_TEST     ((unsigned int) 0x80)
#define LECROY1877_MEMORY   ((unsigned int) 0x40)
#define LECROY1877_ENCOMS   ((unsigned int) 0x4)
#define LECROY1877_LOGICAL  ((unsigned int) 0x2)

/*********************************************************/
/* Define bits in CSR1                                   */
/*********************************************************/
#define LECROY1877_COMMON   ((unsigned int) 0x80000000)
#define LECROY1877_LEAD     ((unsigned int) 0x40000000)
#define LECROY1877_TRAIL    ((unsigned int) 0x20000000)
#define LECROY1877_FCW      24 /* TEST ME!! */
#define LECROY1877_INTERN   ((unsigned int) 0x800000)
#define LECROY1877_ENSPARSE ((unsigned int) 0x100000)
#define LECROY1877_PULSE    18
#define LECROY1877_BURST    16
#define LECROY1877_CST      4

#define LECROY1877_TR5_ENFC  ((unsigned int) 0x8)
#define LECROY1877_TR5_ENFCW ((unsigned int) 0x4)
#define LECROY1877_TR5_ENCOM ((unsigned int) 0x2)
#define LECROY1877_TR7_ENBIP ((unsigned int) 0x1)

/*********************************************************/
/* Define bits in Header Word                            */
/*********************************************************/
#define LECROY1877_COUNT     0
#define LECROY1877_BUFFER    11
#define LECROY1877_PARITY    15
#define LECROY1877_LOG_ADD   16
#define LECROY1877_GEO_ADD   27

/*********************************************************/
/* Define bits in Data Word                              */
/*********************************************************/
#define LECROY1877_DPARITY   26
#define LECROY1877_MTDHIT    24
#define LECROY1877_CHAN      17
#define LECROY1877_EDGE      16
#define LECROY1877_TIME      0

#endif
