/* -*- mode: c++ -*- */
#ifndef __FPGACaenV1495_analysis_h__
#define __FPGACaenV1495_analysis_h__
#include <vector>

#include <data/Analysis.h>
#include <data/DataFormat.h>
#include "DAQ++/DAQid.h"

/**
 * This is the class that performs the analysis of the data sampled by
 * the Monitor process. This process, if enabled, will call
 * Analysis::Fill, which in turn will call
 *    Analysis::data_valid(unsigned long *data) to validate the data
 *                                              chunk
 * and, if the data is valid,
 *    Analysis::Analyze();
 *
 * Analyze expects the data already decoded and stored somehow in the
 * class. The data decoding is performed by the method Analysis::decode
 *
 * Both data_valid and Analyze are virtual methods so you can derive
 * from Analysis and overwrite those methods.
 *
 **/
class FPGACaenV1495_analysis : public Analysis
{
    public:
        FPGACaenV1495_analysis(int nchip, int nchan, const DAQpp::DAQid &id)
          : Analysis(nchip, nchan/nchip)
          , _id(id)
          , _ntrigger(nchan)
          , _coincidence(3)
          , _time_counter(0)
        {
        }

        bool data_valid(unsigned int* data) const;

        /**
         * This is the method called to analyze the data once decoded
         * 
         */
        void Analyze();

        /**
         * This is the method called by the monitoring process to decode
         * the data and leave it ready to be displayed.
         *
         * The only thing it has to do is to fill the pointer returned
         * by the Analysis::Event() and, ask analysis to calculate the
         * pedestals according to the method selected in the VMEDAQ
         * menu. 
         **/
        void decode(unsigned int *data);
        
        
        void init_run();
        void end_run();

  unsigned int get_coinc() const {return _coincidence[0];}
  int get_event_number() const {return ie;}

    private:
        const DAQpp::DAQid _id;
        int _ntrigger;
        std::vector<unsigned int> _coincidence;
        unsigned long long _time_counter;
  int ie;
};

#endif
