#ifndef __FPGACaenV1495_gui_h__
#define __FPGACaenV1495_gui_h__
#include <gui/GladeApp.h>
#include <gui/MonitorWindow.h>

#include "FPGACaenV1495.h"

class FPGACaenV1495_gui : public GladeApp
{
    public:
        FPGACaenV1495_gui(const DAQpp::DAQid &id)
          : _id(id)
        {
        }

        // This is called after the glade file is loaded
        virtual void user_init()
        {
        }

        // The user should fill connect in this routine
        virtual void auto_connect()
        {
        }

        // The user should fill handlers in this routine
        virtual void set_handlers()
        {
        }

        // Any GtkEditable that changes its text will trigger a call to this
        virtual void process_text(const std::string &s, GtkWidget *w)
        {
        }

    protected:
        const DAQpp::DAQid _id;
};

class FPGACaenV1495_monitor : public FPGACaenV1495_gui
{
    public:
        FPGACaenV1495_monitor(const DAQpp::DAQid &id);
        ~FPGACaenV1495_monitor();

        void inLocal();
        void inGlobal();

        // GladeApp methods
        void user_init();
        void auto_connect();
        void set_handlers();

    private:
        int tmout, tmout_stats;
        int g_chip;
        Analysis *_ana;
        bool updating;
        bool has_stats_prop;

        typedef std::set<GtkWidget *> HstList;
        HstList hlist;

        static void on_refresh_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *m);
        static void on_quit_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *m);
        static void on_help_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *m);

        static void on_NewChip(GtkWidget *, FPGACaenV1495_monitor *m);
        static int  on_Update(void *);
        static int  on_UpdateStats(void *);

        void stop_histograms();
        void update_histograms();
        void reset_histograms();

        static GtkWidget *create_hst_signal(FPGACaenV1495_monitor *, char *, char *, char *, int, int);
        static GtkWidget *create_event_display(FPGACaenV1495_monitor *, char *, char *, char *, int, int);
        static GtkWidget *create_hitmap(FPGACaenV1495_monitor *, char *, char *, char *, int, int);

        GtkWidget *create_2D_frame(const std::string &prefix, char *xtit1, char *ytit1, char *xtit2, char *ytit2);

        void resize_hst1d(GtkWidget *, int, double, double, gpointer, double hmin = 0., double hmax = 0.);
        void resize_hst2d(GtkWidget *, int, double, double, int, double, double, gpointer);
};

class FPGACaenV1495_config : public FPGACaenV1495_gui
{
    public:
        FPGACaenV1495_config(const DAQpp::DAQid &id)
          : FPGACaenV1495_gui(id)
        {
        }
        static void on_cancel_button_clicked(GtkWidget *w, FPGACaenV1495_config *m);
        static void on_apply_button_clicked(GtkWidget *w, FPGACaenV1495_config *m);
        static void on_ok_button_clicked(GtkWidget *w, FPGACaenV1495_config *m);

        void user_init();
        void auto_connect()
        {
            SET_CONNECT(on_cancel_button_clicked);
            SET_CONNECT(on_apply_button_clicked);
            SET_CONNECT(on_ok_button_clicked);
        }

    private:
        void update();
        void apply();
};

#endif
