#ifndef __recon_analysis_h__
#define __recon_analysis_h__

#include <data/Analysis.h>
#include <data/DataFormat.h>
//#include "FPGACaenV1495_analysis.h"
//#include "VATAmodule_analysis.h"
//#include "BGOmodule_analysis.h"
#include <map>

/**
 * This is the class that performs the analysis of the data sampled by
 * the Monitor process. This process, if enabled, will call
 * Analysis::Fill, which in turn will call
 *    Analysis::data_valid(unsigned long *data) to validate the data
 *                                              chunk
 * and, if the data is valid,
 *    Analysis::Analyze();
 *
 * Analyze expects the data already decoded and stored somehow in the
 * class. The data decoding is performed by the method Analysis::decode
 *
 * Both data_valid and Analyze are virtual methods so you can derive
 * from Analysis and overwrite those methods.
 *
 **/

class reconModule_analysis : public Analysis
{
    public:
  reconModule_analysis(int,int);

  bool data_valid(unsigned int* data) const
        {
	  //std::cout<<"reconModule_analysis::data_valid "<<std::endl;
            return true;
        }
        
        /**
         * This is the method called to analyze the data once decoded
         * 
         */
        void Analyze();

        /**
         * This is the method called by the monitoring process to decode
         * the data and leave it ready to be displayed.
         *
         * The only thing it has to do is to fill the pointer returned
         * by the Analysis::Event() and, ask analysis to calculate the
         * pedestals according to the method selected in the VMEDAQ
         * menu. 
         **/
        void decode(unsigned int *data){
	  //std::cout<<"reconModule_analysis::decode"<<std::endl;
	}
        void set_angle(double a){angle=a;}
        
        void init_run();
        void end_run(){;}
 private:
	//FPGACaenV1495_analysis *fpga_ana;
	//int fpgaModuleID;
	std::map<int, Analysis *> anaList;
	double angle;
	std::map<int,int> sigMap;
	//std::map<int, BGOmodule_analysis *> bgoModules;
 public:
	static const double pi2;
};

#endif
