/* -*- mode: c++ -*- */
#ifndef __TAmanager__included__
#define __TAmanager__included__

#include <iostream>
#include <vector>
#define TAnCHAN 128

class TAmanager;

int LoadTAvector(const char *file, TAmanager *ta);

class bitset
{
    public:
        int byte_reverse(int val, int nbit) const;
        
    private:
        unsigned long *pb; // Bit string starts here
        int nbits; // Number of bits
        int szSlot; // Internal: size of unsigned int slot
        int nSlot; // number of slots
        int nsTrue; // true number of slots
        int nset; // Number of Bits set
        void cpBits(int, unsigned long *);
        void prt(std::ostream &);

    public:
        friend std::ostream& operator<<(std::ostream&, bitset&);
        // Constructors (int refers always to no. of bits)
        bitset(int nn = TAnCHAN, int szW = 8*sizeof(unsigned long));
        bitset(const bitset&);
        bitset(int, char *);
        bitset(int, unsigned long *);

        // Destructor
        ~bitset();
        // Copy operator
        bitset& operator =(const bitset& y);

        // Operators
        void new_size(int, int);
        bitset * Reset(unsigned long *);
        bitset * Word(int, unsigned long); // Set Word i to value x
        bitset * Set(int); // Sets    i-th bit
        bitset * Set(int val, int start, int width); // Sets to val the byte
        bitset * SetR(int val, int start, int width)
        {
	    return Set(byte_reverse(val,width),start, width);
	}
        // at start with length width
        bitset * Clr(int); // Clears  i-th bit
        bitset * operator!();
        bitset * AllZero();
        int operator[](int) const; // alias for CheckBit

        int Bit(int) const; // Checks  i-th bit
        void BitStr(int, char *); // Set bits according th 0's and 1's in string

        // interface
        unsigned long Word(int x) const;
        unsigned int nWords() const
        {
            return nSlot;
        }

        // Get length of bit stream
        int nBits()
        {
            return nbits;
        }

        // get number of bits to 1
        int nSet()
        {
            return nset;
        }
        unsigned long Byte(int, int) const;

        unsigned long ByteR(int start, int width) const
        {
	   return byte_reverse(Byte(start, width), width);
	}
        void Print(int, int);
        void Dump(); // Dump internals

};

class TAchip : public bitset
{
    protected:
        int nchan;
    public:
        TAchip(int nc = TAnCHAN) :
            nchan(nc)
        {
        }
        TAchip(const TAchip &);
        virtual ~TAchip()
        {
        }

        TAchip &operator=(const TAchip &);

        virtual TAchip *test(int i, bool t);
        virtual TAchip *mask(int i, bool t)
        {
            return this;
        }
        virtual TAchip *trim(int i, int val)
        {
            return this;
        }
        virtual TAchip *Polarity(bool t)
        {
            return this;
        }
        virtual TAchip *EnableGain(bool t)
        {
            return this;
        }
        virtual TAchip *SetGain(int val)
        {
            return this;
        }
        virtual TAchip *Test(bool t)
        {
            return this;
        }

        virtual void Resize(int n_chan) = 0;

        int Nchan()
        {
            return nchan;
        }
        virtual int test(int i)
        {
            //return Bit(nchan -1 - i);
            return Bit(nBits() - nchan - 1 + i);
        }
        virtual int mask(int i)
        {
            return 0;
        }
        virtual int trim(int i)
        {
            return 0;
        }
        virtual int Polarity()
        {
            return 0;
        }
        virtual int GainEnabled()
        {
            return 0;
        }
        virtual int Gain()
        {
            return 0;
        }
        virtual int Test()
        {
            return 0;
        }

        virtual bool Probe_Enable()
        {
            return false;
        }
        virtual bool CC_Enable()
        {
            return false;
        }
        virtual bool N_side()
        {
            return false;
        }
        virtual int Chip_Address()
        {
            return 0;
        }

        virtual TAchip *Probe_Enable(bool t)
        {
            return this;
        }
        virtual TAchip *CC_Enable(bool t)
        {
            return this;
        }
        virtual TAchip *N_side(bool t)
        {
            return this;
        }
        virtual TAchip *Chip_Address(int add)
        {
            return this;
        }

        /*
         * New stuff for GP7
         */
        virtual TAchip * Vrc(bool)
        {
            return this;
        }
        virtual bool Vrc() const
        {
            return false;
        }

        virtual TAchip *Vfsf(bool)
        {
            return this;
        }
        virtual bool Vfsf() const
        {
            return false;
        }

        virtual TAchip *Vfss(bool)
        {
            return this;
        }
        virtual bool Vfss() const
        {
            return false;
        }

        virtual TAchip *Vfp(bool)
        {
            return this;
        }
        virtual bool Vfp() const
        {
            return false;
        }

        virtual TAchip *dac_range(bool)
        {
            return this;
        }
        virtual bool dac_range() const
        {
            return false;
        }

        virtual TAchip *thrs_dac(int)
        {
            return this;
        }
        virtual int thrs_dac() const
        {
            return 0;
        }

        virtual TAchip *calib_gen(bool)
        {
            return this;
        }
        virtual bool calib_gen() const
        {
            return false;
        }

        virtual TAchip *calib_ctrl(bool)
        {
            return this;
        }
        virtual bool calib_ctrl() const
        {
            return false;
        }

        virtual TAchip *calib_dac(int)
        {
            return this;
        }
        virtual int calib_dac() const
        {
            return 0;
        }

        virtual TAchip *vrc_neg(bool)
        {
            return this;
        }
        virtual bool vrc_neg() const
        {
            return false;
        }

        virtual TAchip *slew(bool)
        {
            return this;
        }
        virtual bool slew() const
        {
            return false;
        }

        virtual TAchip *dltc(bool)
        {
            return this;
        }
        virtual bool dltc() const
        {
            return false;
        }

        virtual TAchip *test_channel(bool)
        {
            return this;
        }
        virtual bool test_channel() const
        {
            return false;
        }

        virtual void print(std::ostream &)
        {
        }
        friend std::ostream& operator<<(std::ostream&, TAchip &ch);
};

class VATAGPchip : public TAchip
{
    public:
        VATAGPchip(int nc = 128) :
            TAchip(nc)
        {
            Resize(nc);
        }
        VATAGPchip(const VATAGPchip &);

        void Resize(int n_chan);
        TAchip *mask(int i, bool t);
        TAchip *trim(int i, int val);

        int mask(int i)
        {
            //return Bit(2*nchan - 1 - i);
            return Bit( 17 + 3*nchan + i);
        }
        int trim(int i)
        {
            //return Byte(5*nchan - 3*(i + 1), 3);
            return ByteR( 17 + 3*i, 3);
        }
        int Polarity()
        {
            //return Bit( 5*nchan + 13);
            return Bit( 3);
        }
        int GainEnabled()
        {
            //return Bit( 5*nchan + 12) == 0;
            return Bit( 4);
        }
        int Gain()
        {
            //return Byte( 5*nchan + 10, 2);
            return Byte( 5, 2);
        }
        int Test()
        {
            //return Bit( 5*nchan + 14);
            return Bit( 2);
        }

        bool Probe_Enable()
        {
            //return Bit(5*nchan + 9) ? true : false;
            return Bit( 7) ? true : false;
        }
        bool CC_Enable()
        {
            //return Bit(5*nchan + 16) ? true : false;
            return Bit( 0) ? true : false;
        }
        bool N_side()
        {
            //return Bit(5*nchan + 15) ? true : false;
            return Bit( 1) ? true : false;
        }
        int Chip_Address()
        {
            //return Byte(5*nchan, 9);
            return ByteR( 8, 9);
        }
        TAchip *Probe_Enable(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 9) : Clr(5*nchan + 9));
            return (TAchip *)(t ? Set( 7) : Clr( 7));
        }
        TAchip *CC_Enable(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 16) : Clr(5*nchan + 16));
            return (TAchip *)(t ? Set( 0) : Clr( 0));
        }
        TAchip *N_side(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 15) : Clr(5*nchan + 15));
            return (TAchip *)(t ? Set( 1) : Clr( 1));
        }
        TAchip *Chip_Address(int add)
        {
            //return (TAchip *)Set(add, 5*nchan, 9);
            return (TAchip *)SetR(add, 8, 9);
        }
        TAchip *Polarity(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 13) : Clr(5*nchan + 13));
            return (TAchip *)(t ? Set( 3) : Clr( 3));
        }
        TAchip *EnableGain(bool t)
        {
            //return (TAchip *)(t ? Clr(5*nchan + 12) : Set(5*nchan + 12));
            return (TAchip *)(t ? Set( 4) : Clr( 4));
        }
        TAchip *SetGain(int val)
        {
            //return (TAchip *)Set(val&3, 5*nchan + 10, 2);
            return (TAchip *)Set(val&3, 5, 2);
        }
        TAchip *Test(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 14) : Clr(5*nchan + 14));
            return (TAchip *)(t ? Set( 2) : Clr( 2));
        }

        virtual void print(std::ostream &);
};

class VATAGP3chip : public TAchip
{
    public:
        VATAGP3chip(int nc = 128) :
            TAchip(nc)
        {
            Resize(nc);
        }
        VATAGP3chip(const VATAGP3chip &);

        void Resize(int n_chan);
        TAchip *mask(int i, bool t);
        TAchip *trim(int i, int val);

        //int  mask(int i) { return Bit(2*nchan-1-i);  }
        int mask(int i)
        {
            //return Bit(5*nchan - 1 - i);
            return Bit( 11 + i);
        }
        //      int  trim(int i) { return Byte(5*nchan-3*(i+1),3); }
        int trim(int i)
        {
            //return Byte(4*nchan - 3*(i + 1), 3);
            return ByteR( 11 + nchan + 3*i, 3);
        }
        int Polarity()
        {
            //return Bit( 5*nchan + 7);
            return Bit( 3);
        }
        int GainEnabled()
        {
            //return Bit( 5*nchan + 6);
            return Bit( 4);
        }
        int Gain()
        {
            //return Byte( 5*nchan + 4, 2);
            return Byte( 5, 2);
        }
        int Test()
        {
            //return Bit( 5*nchan + 8);
            return Bit( 2);
        }

        bool CC_Enable()
        {
            //return Bit(5*nchan + 10) ? true : false;
            return Bit( 0) ? true : false;
        }
        bool N_side()
        {
            //return Bit(5*nchan + 9) ? true : false;
            return Bit( 1) ? true : false;
        }
        int Chip_Address()
        {
            //return Byte(5*nchan, 4);
            return ByteR( 7, 4);
        }

        TAchip *CC_Enable(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 10) : Clr(5*nchan + 10));
            return (TAchip *)(t ? Set( 0) : Clr( 0));
        }
        TAchip *N_side(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 9) : Clr(5*nchan + 9));
            return (TAchip *)(t ? Set( 1) : Clr( 1));
        }
        TAchip *Chip_Address(int add)
        {
            //return (TAchip *)Set(add, 5*nchan, 4);
            return (TAchip *)SetR(add, 7, 4);
        }
        TAchip *Polarity(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 7) : Clr(5*nchan + 7));
            return (TAchip *)(t ? Set( 3) : Clr( 3));
        }
        TAchip *EnableGain(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 6) : Clr(5*nchan + 6));
            return (TAchip *)(t ? Set( 4) : Clr( 4));
        }
        TAchip *SetGain(int val)
        {
            //return (TAchip *)Set(val&3, 5*nchan + 4, 2);
            return (TAchip *)Set(val&3, 5, 2);
        }
        TAchip *Test(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 8) : Clr(5*nchan + 8));
            return (TAchip *)(t ? Set( 2) : Clr( 2));
        }

        virtual void print(std::ostream &);
};

class VATAGP3pchip : public VATAGP3chip
{
  // Added by Vladimir on 15th of Dec., 2009
    private:
        static int nHeader;
  // --------------------------------------
    public:
        VATAGP3pchip(int nc = 128) :
            VATAGP3chip(nc)
        {
            Resize(nc);
        }
        VATAGP3pchip(const VATAGP3pchip &);

        void Resize(int n_chan);
        TAchip *mask(int i, bool t);
        TAchip *trim(int i, int val);

        int Polarity()
        {
            //return Bit( 5*nchan + 6);
            return Bit( 3);
        }
        int GainEnabled()
        {
            return 0;
        }
        int Gain()
        {
            return 0;
        }
        int Test()
        {
            //return Bit( 5*nchan + 7);
            return Bit( 2);
        }
        bool CC_Enable()
        {
            //return Bit(5*nchan + 9) ? true : false;
            return Bit( 0) ? true : false;
        }
        bool N_side()
        {
            //return Bit(5*nchan + 8) ? true : false;
            return Bit( 1) ? true : false;
        }
        int Chip_Address()
        {
            //return Byte(5*nchan, 4);
            //return ByteR( 4, 4); Commented by Vladimir on 15th of Dec., 2009
	    	return ByteR(5,4);  // Added by Vladimir on 15th of Dec., 2009
        }
        int mask(int i)
        {
            //return Bit(5*nchan - 1 - i);
            //return Bit( 8 + i);       Commented by Vladimir on 15th of Dec., 2009
	    	return Bit(nHeader + i); // Added by Vladimir on 15th of Dec., 2009
        }
        int trim(int i)
        {
            //return Byte(4*nchan - 3*(i + 1), 3);
            //return ByteR( 8 + nchan + 3*i, 3);      Commented by Vladimir on 15th of Dec., 2009
	    	return ByteR(nHeader + nchan + 3*i,3); // Added by Vladimir on 15th of Dec., 2009
        }

        TAchip *CC_Enable(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 9) : Clr(5*nchan + 9));
            return (TAchip *)(t ? Set( 0) : Clr( 0));
        }
        TAchip *N_side(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 9) : Clr(5*nchan + 9));
            return (TAchip *)(t ? Set( 1) : Clr( 1));
        }
        TAchip *Test(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 7) : Clr(5*nchan + 7));
            return (TAchip *)(t ? Set( 2) : Clr( 2));
        }
        TAchip *Polarity(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 6) : Clr(5*nchan + 6));
            return (TAchip *)(t ? Set( 3) : Clr( 3));
        }
        TAchip *EnableGain(bool t)
        {
            return this;
        }
        TAchip *SetGain(int val)
        {
            return this;
        }
        TAchip *Chip_Address(int add)
        {
            //return (TAchip *)Set(add, 5*nchan, 4);
            //return (TAchip *)SetR(add, 4, 4); Commented by Vladimir on 15th of Dec., 2009
	    	return (TAchip *)Set(add, 5, 4); // Added by Vladimir on 15th of Dec., 2009
        }
};

class VATAGP5chip : public TAchip
{
    public:
        VATAGP5chip(int nc=128) : TAchip(nc)
        {
            Resize (nc);
        }
        void Resize(int nchan);
        
        bool    CC_Enable() { return Bit(0) ? true : false; }
        TAchip *CC_Enable(bool t) { return (TAchip *)(t ? Set(0) : Clr(0)); }
        
        TAchip *N_side(bool z) { return (TAchip *)(z ? Set(1) : Clr(1)); }
        bool    N_side() { return Bit(1); }
        
        TAchip *Test(bool z) { return (TAchip *)(z ? Set(2) : Clr(2)); }
        int Test() { return Bit(2); }
        
        TAchip *Polarity(bool z) { return (TAchip *)(z ? Set(3) : Clr(3)); }
        int Polarity() { return Bit(3); }
        
        TAchip *SetGain(int val) { return (TAchip *)Set(val&3, 4, 2);}
        int Gain() { return Byte(4, 2);}
        
        int Chip_Address()
        {
            return ByteR(6, 4);
        }
        TAchip *Chip_Address(int add)
        {
            return (TAchip *)SetR(add, 6, 4);
        }
        void print(std::ostream &);
};

class VATAGP7chip : public TAchip
{
    public:
        VATAGP7chip(int nc=128) :
            TAchip(nc)
        {
            Resize(nc);
        }

        void Resize(int nchan);

        TAchip * Vrc(bool z)
        {
            return (TAchip *)(z ? Set(0) : Clr(0));
        }
        bool Vrc() const
        {
            return Bit(0);
        }

        TAchip *Vfsf(bool z)
        {
            return (TAchip *)(z ? Set(1) : Clr(1));
        }
        bool Vfsf() const
        {
            return Bit(1);
        }

        TAchip *Vfss(bool z)
        {
            return (TAchip *)(z ? Set(2) : Clr(2));
        }
        bool Vfss() const
        {
            return Bit(2);
        }

        TAchip *Vfp(bool z)
        {
            return (TAchip *)(z ? Set(3) : Clr(3));
        }
        bool Vfp() const
        {
            return Bit(3);
        }

        TAchip *CC_Enable(bool z)
        {
            return (TAchip *)(z ? Set(4) : Clr(4));
        }
        bool CC_Enable()
        {
            return Bit(4);
        }

        TAchip *N_side(bool z)
        {
            return (TAchip *)(z ? Set(5) : Clr(5));
        }
        bool N_side()
        {
            return Bit(5);
        }

        TAchip *Test(bool z)
        {
            return (TAchip *)(z ? Set(6) : Clr(6));
        }
        int Test()
        {
            return Bit(6);
        }

        TAchip *Polarity(bool z)
        {
            return (TAchip *)(z ? Set(7) : Clr(7));
        }
        int Polarity()
        {
            return Bit(7);
        }

        TAchip *Chip_Address(int add)
        {
            return (TAchip *)(SetR(add, 8, 4));
        }
        int Chip_Address()
        {
            return ByteR(8, 4);
        }

        TAchip *calib_gen(bool z)
        {
            return (TAchip *)(z ? Set(12) : Clr(12));
        }
        bool calib_gen() const
        {
            return Bit(12);
        }

        TAchip *calib_ctrl(bool z)
        {
            return (TAchip *)(z ? Set(13) : Clr(13));
        }
        bool calib_ctrl() const
        {
            return Bit(13);
        }

        TAchip *vrc_neg(bool z)
        {
            return (TAchip *)(z ? Set(14) : Clr(14));
        }
        bool vrc_neg() const
        {
            return Bit(14);
        }

        TAchip *slew(bool z)
        {
            return (TAchip *)(z ? Set(16) : Clr(16));
        }
        bool slew() const
        {
            return Bit(16);
        }

        TAchip *dac_range(bool z)
        {
            return (TAchip *)(z ? Set(17) : Clr(17));
        }
        bool dac_range() const
        {
            return Bit(17);
        }

        
        TAchip *dltc(bool z)
        {
            return (TAchip *)(z ? Set(18) : Clr(18));
        }
        bool dltc() const
        {
            return Bit(18);
        }

        TAchip *mask(int ichan, bool z)
        {
            int b = 19 + ichan;
            return (TAchip *)(z ? Set(b) : Clr(b));
        }
        int mask(int ichan)
        {
            return Bit( 19 + ichan);
        }

        TAchip *trim(int ichan, int val)
        {
            int b = 147 + 4*ichan;
            return (TAchip *)(SetR(val, b, 4));
        }
        int trim(int ichan)
        {
            return ByteR( 147 + 4*ichan, 4);
        }

        TAchip *test_channel(bool z)
        {
            return (TAchip *)(z ? Set(659) : Clr(659));
        }
        bool test_channel() const
        {
            return Bit(659);
        }

        TAchip *Probe_Enable(bool z)
        {
            return (TAchip *)(z ? Set(659) : Clr(659));
        }
        bool Probe_Enable()
        {
            return Bit(659);
        }

        TAchip *test(int ichan, bool x)
        {
            int b = 660 + ichan;
            return (TAchip *)(x ? Set(b) : Clr(b) );
        }
        int test(int ichan)
        {
            int b = 660 + ichan;
            return Bit(b);
        }
        TAchip *thrs_dac(int val)
        {
            return (TAchip *)(SetR(val, 917, 5));
        }
        int thrs_dac() const
        {
            return ByteR(917, 5);
        }

        TAchip *calib_dac(int val)
        {
            return (TAchip *)(SetR(val, 922, 4));
        }
        int calib_dac() const
        {
            return ByteR(922, 4);
        }

        void print(std::ostream &);

};

class TA32chip : public TAchip
{
    public:
        TA32chip(int nc = 32) :
            TAchip(nc)
        {
            Resize(nc);
        }
        TA32chip(const TA32chip &);

        void Resize(int n_chan);
        TAchip *mask(int i, bool t);
        TAchip *trim(int i, int val);

        int mask(int i)
        {
            //return Bit(5*nchan - 1 - i);
            return Bit( 5 + i);
        }
        int trim(int i)
        {
            //return Byte(4*nchan - 3*(i + 1), 3);
            return ByteR( 5 + nchan + 3*i, 3);
        }

        int Polarity()
        {
            //return Bit( 5*nchan);
            return Bit( 4);
        }
        int GainEnabled()
        {
            //return Bit( 5*nchan + 1) == 0;
            return Bit( 3) == 0;
        }
        int Gain()
        {
            //return Byte( 5*nchan + 2, 2);
            return Byte( 1, 2);
        }
        int Test()
        {
            //return Bit( 5*nchan + 4);
            return Bit( 0);
        }

        TAchip *Polarity(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan) : Clr(5*nchan));
            return (TAchip *)(t ? Set( 4) : Clr( 4));
        }
        TAchip *EnableGain(bool t)
        {
            //return (TAchip *)(t ? Clr(5*nchan + 1) : Set(5*nchan + 1));
            return (TAchip *)(t ? Clr( 3) : Set( 3));
        }
        TAchip *SetGain(int val)
        {
            //return (TAchip *)Set(val&3, 5*nchan + 2, 2);
            return (TAchip *)Set(val&3, 1, 2);
        }
        TAchip *Test(bool t)
        {
            //return (TAchip *)(t ? Set(5*nchan + 4) : Clr(5*nchan + 4));
            return (TAchip *)(t ? Set( 0) : Clr( 0));
        }
        virtual void print(std::ostream &);
};

class TAmanager
{
    public:
        enum ChipType
        {   chipTA, chipGP, chipGP3, chipGP3p, chipGP5, chipGP7, last_type};
    private:
        int nchip;
        int nchan;
        int nall;
        ChipType type;
        typedef std::vector<TAchip *> ChipList;
        ChipList chips;
        bitset *all;

        void clear();
        void cpy(const TAmanager &);
    public:
        TAmanager(ChipType tp = chipGP3, int n_chip = 1);
        ~TAmanager();
        TAmanager &operator=(const TAmanager &);
        TAmanager(const TAmanager &);
        void Resize(int);
        int Nchip() const
        {
            return nchip;
        }
        int Nchan() const
        {
            return chips.empty() ? 0 : chips[0]->Nchan();
        }
        ChipType GetChipType() const
        {
            return type;
        }
        void SetChipType(ChipType tp)
        {
            type = tp;
            Resize(nchip);
        }
        TAchip *chip(int);
        bitset *TAvector();
        int Vsize()
        {
            return (all ? all->nWords() : 0);
        }

        friend std::ostream& operator<<(std::ostream&, TAmanager&);

};
inline TAchip *TAmanager::chip(int i)
{
    if (chips.empty() )
        return 0;

    int nch = chips.size();
    if (i > nch || i < 0)
        return 0;

    return chips[i];
}
#endif
