/* -*- mode: c++ -*- */
#ifndef __TDCCaen_module__h__
#define __TDCCaen_module_h__
/**
 * This is a module for controling a TDC Caen 488 
 *
 **/
#include <DAQ++/Parameter.h>
#include <daq/GenModule.h>
#include <tdc488.h>

class TDCCaen488_module
   : public GenModule,
     public DAQpp::ParameterCollection<TDCCaen488_module>
{
   private:
      tdc488 tdc;
   public:

      /** Constructor
       *  The constructor needs 2 parameters:
       *  id: the Module id
       *  address: the address (VME, or whatever protocol) to
       *           communicate with the module
       *
       *  Note: although DAQ++ allows for any type of identifier,
       *  however VMEDAQ only handles properly integer IDs. The only
       *  reason for that is that it needs integer values to save them
       *  properly in the data file
       *
       **/
      TDCCaen488_module( const DAQpp::DAQid &id,unsigned long address);
      
      /** This is the method called by the RunManager to know if there
       * is data ready in the module. If the answer if true, then it
       * will try to read the module. If there is no way to know a
       * priory about the availavility of data, one can defer the
       * decision until getData
       *
       **/
      bool hasData();

      /** This is the method called to retrieve the data. If hasData
       * could not figure out hte right answer, one can still say here
       * that there is no dat aat all by returning a NULL pointer and
       * setting size to 0.
       *
       * The memory to store the data should be allocated with the
       * allocate method. See the DAQ++ documentation...
       *
       **/
      char *getData(int &size);

      int PrepareForRun ();

      int Start (int = 0);

      int Stop (int = 0);

      int Pause ();

      int Continue ();

      int Reset ();

      int GetReady ();

      int Trigger () { return 0; };
      void flush () {};

      // From GenModule
      void set_header_description(unsigned int &data);
      unsigned long get_data_type() const;
      unsigned int *write_config(int &size);
      
      // Control methods
      void change_low_threshold(const int &);
      void change_high_threshold(const int &);
      void change_range(const double &);
      void change_channel_mask(const int &);
      void change_daq_mode(const int &);
      void change_buffer_depth(const int &);
};

#endif
